/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.colors.ColorBars;
import fr.amap.commons.util.debug.DebugLevel;
import fr.amap.commons.util.debug.Debugger;
import fr.amap.commons.util.locales.AvailablesLocales;
import fr.amap.commons.util.properties.ProjectProperties;
import fr.amap.commons.util.swing.FrameView;
import fr.amap.commons.util.swing.JVerticalPanel;
import fr.amap.lidar.amapvox.nephos3d.util.Skyboxes;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.IOException;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.ScrollPaneConstants;
import fr.amap.lidar.amapvox.visualization.controllers.OptionsController;
import fr.amap.lidar.amapvox.visualization.views.colorbar.ColorbarComboboxRenderer;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.function.Consumer;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.border.LineBorder;
import net.jcip.annotations.GuardedBy;
import net.jcip.annotations.ThreadSafe;
import org.jdesktop.swingx.JXButton;
import org.jdesktop.swingx.JXCollapsiblePane;
import org.jdesktop.swingx.JXComboBox;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.JXTitledPanel;
import org.jdesktop.swingx.combobox.EnumComboBoxModel;
import org.jdesktop.swingx.painter.MattePainter;

/**
 * Options view of the project.
 */
@ThreadSafe
public class OptionsView extends FrameView {

    private final OptionsController controller;

    private JXCollapsiblePane collapsiblePane;
    private JXTitledPanel titledPanel;
    private JScrollPane scrollPane;
    
    private JVerticalPanel editablesPanel;

    private JXLabel localeLabel;
    private JXComboBox localeComboBox;

    private JXLabel backgroundLabel;
    private JXComboBox backgroundComboBox;

    private JXLabel colorbarLabel;
    private JXComboBox colorbarComboBox;
    
    private JXLabel shapeLabel;
    private JXComboBox shapeComboBox;

    private JXLabel voxelSizeLabel;
    private JSlider voxelSizeSlider;
    
    private JXLabel drawFastLabel;
    
    @GuardedBy("this")
    private JCheckBox drawFastCheckbox;
    
    private JXLabel reflectionLabel;
    
    @GuardedBy("this")
    private JCheckBox reflectionCheckbox;
    
    private JXLabel gridLabel;
    private JCheckBox gridCheckbox;
    
    private JXLabel boundingboxLabel;
    private JCheckBox boundingboxCheckbox;

    private JXPanel axisLegendaryPanel;
    private JXLabel axisLegendaryLabel;

    private final CopyOnWriteArrayList<Consumer<Color[]>> onChangeColorBarCallback;

    /**
     * Creates new form RenderingFrame
     * @throws java.lang.Exception
     */
    public OptionsView() throws Exception {
        super();
        this.controller = new OptionsController();
        this.onChangeColorBarCallback = new CopyOnWriteArrayList();
    }

    public final OptionsController getController() {
        return controller;
    }

    public final void updateInitialProperties() {
        controller.changeEnableReflection(reflectionCheckbox.isSelected());
        controller.changeVoxelSize(voxelSizeSlider.getValue());
        controller.setShape(shapeComboBox.getSelectedIndex());
        synchronized(this) {
            drawFastCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
            reflectionCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
        }
        controller.setHideGrid(!gridCheckbox.isSelected());
        controller.setHideBoundingbox(!boundingboxCheckbox.isSelected());
    }

    public final synchronized void addOnChangeColorBarCallback(
            final Consumer<Color[]> onChangeColorBarCallback) {
        this.onChangeColorBarCallback.add(onChangeColorBarCallback);
    }

    @Override
    protected void initContainer() throws Exception {
        setLocationRelativeTo(getParent());
    }

    @Override
    protected void initComponents() throws Exception {
        /* Create the collapsible panel */
        collapsiblePane = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT);
        collapsiblePane.setAnimated(false);
        collapsiblePane.setLayout(new BorderLayout());

        final Action toggleAction = collapsiblePane.getActionMap().
                get(JXCollapsiblePane.TOGGLE_ACTION);

        toggleAction.putValue(JXCollapsiblePane.COLLAPSE_ICON, loadIcon("/icons/right.png"));
        toggleAction.putValue(JXCollapsiblePane.EXPAND_ICON, loadIcon("/icons/left.png"));

        final JXButton toggle = new JXButton(toggleAction);
        toggle.setText(null);
        toggle.setPreferredSize(new Dimension(20, 20));

        add(collapsiblePane, BorderLayout.CENTER);
        add(toggle, BorderLayout.WEST);

        /* Create title panel */
        titledPanel = new JXTitledPanel();
        titledPanel.getContentContainer().setLayout(new BorderLayout());
        addLocaleCallback((locale) -> {
            titledPanel.setTitle(bundle.getString("optionsView_Title"));
        });
        collapsiblePane.getContentPane().add(titledPanel);

        /* Create scroll pane */
        scrollPane = new JScrollPane();
        scrollPane.setHorizontalScrollBarPolicy(
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPane.setVerticalScrollBarPolicy(
                ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        titledPanel.getContentContainer().add(scrollPane, BorderLayout.CENTER);
        
        /* Create editables panel */
        editablesPanel = new JVerticalPanel();
        scrollPane.setViewportView(editablesPanel);
        
        /* Create the locale components */
        localeLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            localeLabel.setText(bundle.getString("optionsView_localeLabel") + " :");
        });

        localeComboBox = new JXComboBox();
        final ComboBoxModel model = new EnumComboBoxModel(AvailablesLocales.class);
        localeComboBox.setModel(model);
        addLocaleCallback((locale) -> {
            localeComboBox.setSelectedItem(AvailablesLocales.getFromLocale(locale));
            localeComboBox.setToolTipText(bundle.getString("optionsView_localeCombobox_tooltip"));
        });
        localeComboBox.addActionListener((java.awt.event.ActionEvent evt) -> {
            if (parentView != null) {
                final AvailablesLocales locale = ((AvailablesLocales) localeComboBox.getSelectedItem());
                parentView.setLocale(locale.getLocale());
                ProjectProperties.setProperty("language", locale.name());
            }
        });
        
        editablesPanel.addLine(localeLabel, localeComboBox);

        /* Create background components */
        backgroundLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            backgroundLabel.setText(bundle.getString("optionsView_backgroundLabel") + " :");
        });

        backgroundComboBox = new JXComboBox(Skyboxes.getSkyNames());
        backgroundComboBox.setSelectedItem(ProjectProperties.getProperty("background", "Simple"));
        addLocaleCallback((locale) -> {
            backgroundComboBox.setToolTipText(bundle.getString("optionsView_backgroundComboBox_tooltip"));
        });
        backgroundComboBox.addActionListener((java.awt.event.ActionEvent evt) -> {
            try {
                /* Change the background of the scene */
                controller.changeBackground((String) backgroundComboBox.getSelectedItem());
            } catch (IOException ex) {
                Debugger.debugInterface(ex, DebugLevel.ERROR, parentView);
            }
        });
        
        editablesPanel.addLine(backgroundLabel, backgroundComboBox);
        
        /* Create colorbar components */
        colorbarLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            colorbarLabel.setText(bundle.getString("optionsView_colorbarLabel") + " :");
        });

        final Integer[] intArray = new Integer[ColorBars.getColorBarsNames().length];
        for (int i = 0; i < intArray.length; i++) {
            intArray[i] = i;
        }
        colorbarComboBox = new JXComboBox(intArray);
        colorbarComboBox.setRenderer(new ColorbarComboboxRenderer());
        colorbarComboBox.setSelectedIndex(ColorBars.getIndexOf(
                ProjectProperties.getProperty("colorBar", "Equatorial")));
        addLocaleCallback((locale) -> {
            colorbarComboBox.setToolTipText(bundle.getString("optionsView_colorbarComboBox_tooltip"));
            final String colorBarName = ProjectProperties.getProperty("colorBar", "Equatorial");
            for (Consumer<Color[]> consumer : onChangeColorBarCallback) {
                consumer.accept(ColorBars.getColorBar(colorBarName));
            }
        });
        colorbarComboBox.addActionListener((e) -> {
            final String colorBarName = ColorBars.getColorBarsNames()[colorbarComboBox.getSelectedIndex()];
            
            /* Change the color bar of the voxel space */
            controller.changeColorBar(colorBarName);

            for (Consumer<Color[]> consumer : onChangeColorBarCallback) {
                consumer.accept(ColorBars.getColorBar(colorBarName));
            }
        });

        editablesPanel.addLine(colorbarLabel, colorbarComboBox);
        
        /* Create shape components */
        shapeLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            shapeLabel.setText(bundle.getString("optionsView_shapeLabel") + " :");
        });
        
        shapeComboBox = new JXComboBox();
        addLocaleCallback((locale) -> {
            final ComboBoxModel shapeModel = new DefaultComboBoxModel(new String[] {
                bundle.getString("optionsView_shapeComboBox_pixel"),
                bundle.getString("optionsView_shapeComboBox_cube"),
            });
            shapeComboBox.setModel(shapeModel);
            shapeComboBox.setSelectedIndex(Integer.valueOf(ProjectProperties.getProperty("shape", "1")));
            shapeComboBox.setToolTipText(bundle.getString("optionsView_shapeComboBox_tooltip"));
        });
        shapeComboBox.addItemListener((e) -> {
            controller.setShape(shapeComboBox.getSelectedIndex());
            drawFastCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
            reflectionCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
        });
        
        editablesPanel.addLine(shapeLabel, shapeComboBox);

        /* Create voxel size components */
        voxelSizeLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            voxelSizeLabel.setText(bundle.getString("optionsView_voxelSizeSlider") + " :");
        });

        voxelSizeSlider = new JSlider(1, 100);
        voxelSizeSlider.setPreferredSize(new Dimension(100, 30));
        voxelSizeSlider.setValue(Integer.valueOf(
                ProjectProperties.getProperty("voxelSize", "100")));
        addLocaleCallback((locale) -> {
            voxelSizeSlider.setToolTipText(bundle.getString("optionsView_voxelSizeSlider_tooltip"));
        });
        voxelSizeSlider.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                controller.changeVoxelSize(voxelSizeSlider.getValue());
            }
        });

        editablesPanel.addLine(voxelSizeLabel, voxelSizeSlider);
        
        /* Create draw fast components */
        drawFastLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            drawFastLabel.setText(bundle.getString("optionsView_drawFastCheckbox"));
        });
        
        drawFastCheckbox = new JCheckBox();
        drawFastCheckbox.setSelected(Boolean.valueOf(
                ProjectProperties.getProperty("drawFastEnabled", "true")));
        addLocaleCallback((locale) -> {
            drawFastCheckbox.setToolTipText(bundle.getString("optionsView_drawFastCheckbox_tooltip"));
        });
        drawFastCheckbox.addActionListener((e) -> {
            controller.changeDrawFast(drawFastCheckbox.isSelected());
        });

        editablesPanel.addLine(drawFastLabel, drawFastCheckbox);

        /* Create reflection components */
        reflectionLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            reflectionLabel.setText(bundle.getString("optionsView_reflectionCheckbox"));
        });
        
        reflectionCheckbox = new JCheckBox();
        addLocaleCallback((locale) -> {
            reflectionCheckbox.setToolTipText(bundle.getString("optionsView_reflectionCheckbox_tooltip"));
        });
        reflectionCheckbox.addActionListener((e) -> {
            controller.changeEnableReflection(reflectionCheckbox.isSelected());
        });
        reflectionCheckbox.setSelected(Boolean.valueOf(
                ProjectProperties.getProperty("reflectionsEnabled", "true")));
        
        editablesPanel.addLine(reflectionLabel, reflectionCheckbox);
        
        /* Create grid panel */
        gridLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            gridLabel.setText(bundle.getString("optionsView_gridCheckbox"));
        });
        
        gridCheckbox = new JCheckBox();
        gridCheckbox.setSelected(!Boolean.valueOf(
                ProjectProperties.getProperty("hideGrid", "false")));
        addLocaleCallback((locale) -> {
            gridCheckbox.setToolTipText(bundle.getString("optionsView_gridCheckbox_tooltip"));
        });
        gridCheckbox.addActionListener((e) -> {
            controller.setHideGrid(!gridCheckbox.isSelected());
        });
        
        editablesPanel.addLine(gridLabel, gridCheckbox);
        
        /* Create boundingbox panel */
        boundingboxLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            boundingboxLabel.setText(bundle.getString("optionsView_boundingboxCheckbox"));
        });
        
        boundingboxCheckbox = new JCheckBox();
        boundingboxCheckbox.setSelected(!Boolean.valueOf(
                ProjectProperties.getProperty("hideBoundingbox", "false")));
        addLocaleCallback((locale) -> {
            boundingboxCheckbox.setToolTipText(bundle.getString("optionsView_boundingboxCheckbox_tooltip"));
        });
        boundingboxCheckbox.addActionListener((e) -> {
            controller.setHideBoundingbox(!boundingboxCheckbox.isSelected());
        });
       
        editablesPanel.addLine(boundingboxLabel, boundingboxCheckbox);
        
        /* Create axis legendary label */
        axisLegendaryPanel = new JXPanel(new CardLayout(2, 2));
        titledPanel.getContentContainer().add(axisLegendaryPanel, BorderLayout.PAGE_END);

        axisLegendaryLabel = new JXLabel(loadIcon("/icons/axisLegendary.png"));
        axisLegendaryLabel.setBackgroundPainter(new MattePainter(
                new Color((float) 208 / 255, (float) 214 / 255, (float) 217 / 255)));
        axisLegendaryLabel.setBorder(new LineBorder(Color.GRAY));
        axisLegendaryPanel.add(axisLegendaryLabel);
//        editablesPanel.addLine(axisLegendaryPanel);

        pack();
    }

}
